;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                  ;;
;;  IF YOU USE THIS PROGRAM, YOU AGREE                              ;;
;;  TO DO SO AT YOUR OWN RISK.                                      ;;
;;                                                                  ;;
;;  File name:   slay.lsp                                           ;;
;;  Version:     3                                                  ;;
;;  Author:      Bill Oneal  (Compuserve 76600,2214)                ;;
;;  Date:        14 August 1994                                     ;;
;;  Purpose:     Defines command SLAY (Sort by LAYer).              ;;
;;  Target:      AutoCAD R12c3 for Extended DOS                     ;;
;;                                                                  ;;
;;  The SLAY command sorts drawing entities by lexical order of     ;;
;;  layer name.                                                     ;;
;;                                                                  ;;
;;  Entity order can determine which entities obscure which other   ;;
;;  entities on the display and in output.                          ;;
;;  Note that you must make the appropriate choices in              ;;
;;  Settings / Selection Settings / Entity Sort Method.             ;;
;;                                                                  ;;
;;  A convenient way to order layer names is to prefix them         ;;
;;  with numbers.  Use leading zeroes, so 002 will precede 010.     ;;
;;                                                                  ;;
;;  SLAY works by copying and erasing selection sets of entities.   ;;
;;  This fact has some important consequences.                      ;;
;;                                                                  ;;
;;  1. SLAY affects only entities in the current space.  You can    ;;
;;     use it in model space or paper space.                        ;;
;;                                                                  ;;
;;  2. SLAY does not affect entities on locked layers.  Entities    ;;
;;     on locked layers will precede other entities.  If you want   ;;
;;     all layers in order, you must unlock layers, run SLAY, then  ;;
;;     lock layers again.                                           ;;
;;                                                                  ;;
;;  3. SLAY does not maintain the order of entities on a given      ;;
;;     layer.  If you are configured to sort for entity selection,  ;;
;;     it _probably_ always reverses them.  In that case, calling   ;;
;;     SLAY twice will _probably_ maintain the order of entities    ;;
;;     on a given layer.                                            ;;
;;                                                                  ;;
;;  4. SLAY does not affect the order of entities within an XREF.   ;;
;;                                                                  ;;
;;  5. On most systems, entities will disappear layer-by-layer      ;;
;;     as SLAY works.  Don't panic.  They will reappear.            ;;
;;                                                                  ;;
;;  If you use a display list driver, you probably will have to     ;;
;;  REGEN or REGENALL to see the true order of entities on the      ;;
;;  display.  Remember that you must select Regens in               ;;
;;  Entity Sort Methods.                                            ;;
;;                                                                  ;;
;;  Portability:                                                    ;;
;;  Not tested with any other release or platform.                  ;;
;;  This version of SLAY expects entities to have group 67 data     ;;
;;  (paper or model space).  It also expects to read TILEMODE.      ;;
;;  It probably will not work unmodified with AutoCAD releases      ;;
;;  prior to R11.                                                   ;;
;;                                                                  ;;
;;  Revision History:                                               ;;
;;                                                                  ;;
;;  Version 1: 16 March 1994                                        ;;
;;  Version 2: 25 May 1994                                          ;;
;;    No longer gives error message "Unknown Command" while sorting ;;
;;    a drawing with both paper space and model space entities.     ;;
;;    (Version 1 functioned correctly in spite of that message.)    ;;
;;  Version 3: 14 August 1994                                       ;;
;;    Replaced redraw with redrawall for the case of model space.   ;;
;;                                                                  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun C:SLAY

    (/
     fail         ; function to exit with an error message
     cmdecho      ; setting of CMDECHO   at start
     regenmode    ; setting of REGENMODE at start
     tilemode     ; setting of TILEMODE  (will not change)
     cvport       ; setting of CVPORT    (will not change)
     cspace       ; current space (0 for model, 1 for paper)
     lay          ; a layer from the layer symbol table
     layname      ; a layer name
     laynamelist  ; list of all the layer names
     ss           ; a selection set
     filter       ; the filter used to select ss
     psvname      ; paper space viewport entity name
    )

  ;-------------------------------------------------------------------
  ; Define function fail to exit with an error message

    (defun fail (failmsg)
        (print failmsg)
        (exit)
    )

  ;-------------------------------------------------------------------
  ; Get system variables

    (setq cmdecho   (getvar "CMDECHO"  )
          regenmode (getvar "REGENMODE")
          tilemode  (getvar "TILEMODE" )
          cvport    (getvar "CVPORT"   )
    )

  ;-------------------------------------------------------------------
  ; Get current space

    (if (and (equal tilemode 0) (equal cvport 1))
        (setq cspace 1)
        (setq cspace 0)
    )

  ;-------------------------------------------------------------------
  ; Find paper space viewport

    ; Initially, the paper space viewport name should
    ; be undefined, in case we cannot find one.

    (setq psvname nil)

    ; Define a selection set filter that accepts only
    ; entity type = "VIEWPORT" and viewport ID = 1.
    ; Use the filter to get a selection set.

    (setq filter '(( 0 . "VIEWPORT") (69 . 1)))
    (setq ss (ssget "X" filter))

    ; There should be at most one paper space viewport.

    (if ss
        (if (equal (sslength ss) 1)
            (setq psvname (ssname ss 0))
        )
    )


    ; Undefine symbols that will be reused.

    (setq filter nil)
    (setq ss nil)

  ;-------------------------------------------------------------------
  ; Get a list of all the layer names

    ; Layer name list begins empty.
    ; Get the first layer from the layer table.
    ; When there are no more layers, tblnext will return nil.
    ; For each layer:
    ;     Get the layer name from group 2 of the layer information.
    ;     Add the layer name to the list of layer names.
    ;     Get the next layer from the layer table.

    (setq laynamelist  '())
    (setq lay (tblnext "LAYER" 1))
    (while lay
        (setq layname  (cdr (assoc  2 lay)))
        (setq laynamelist  (cons layname laynamelist))
        (setq lay (tblnext "LAYER"))
    )

    (if (not laynamelist) (fail "This drawing has no layers?!"))

    ; Undefine symbols that will be reused.

    (setq lay nil)
    (setq layname nil)

  ;-------------------------------------------------------------------
  ; Sort the list of all the layer names

    ; If the function that sorts lists of strings is not yet defined,
    ; load the file that defines it.

    (if (not (car (atoms-family 0 '("acad_strlsort"))))
        (xload "acadapp")
    )

    (setq laynamelist (acad_strlsort laynamelist))

  ;-------------------------------------------------------------------
  ; Sort the entities

    (setvar "REGENMODE" 0)

    (foreach layname laynamelist

        ; Define a selection set filter that accepts only
        ; layer = layname and space = cspace.
        ; Use the filter to get a selection set.

        (setq filter (list (cons 8 layname) (cons 67 cspace)))
        (setq ss (ssget "X" filter))

        ; Delete the paper space viewport from the selection set.
        ; It should be only in paper space on layer "0".
        ; However, no harm is done if we attempt to delete it
        ; from every selection set.
        ; Just make sure the paper space viewport name has been
        ; defined and the selection set is not already empty.

        (if (and psvname ss)
            (if (not (equal (sslength ss) 0))
                (ssdel psvname ss)
            )
        )

        ; If the selection set is still not empty after filtering
        ; for layer and space, and deleting the paper space viewport:
        ; 1.  copy the selected entities with zero displacement
        ; 2.  erase the original entities.

        (if ss
            (if (not (equal (sslength ss) 0))
                (progn
                    (setvar "CMDECHO" 0)
                    (command "_copy" ss "" '(0 0) "")
                    (command "_erase" "_p" "")
                    (setvar "CMDECHO" cmdecho)
                )
            )
        )
    )

    (setvar "CMDECHO" 0)
    (if (equal cspace 0)           ; if in model space
        (command "_redrawall")         ; redraw all viewports
        (command "_redraw")            ; else redraw current viewport
    )
    (setvar "CMDECHO" cmdecho)

    (setvar "REGENMODE" regenmode)

  ;-------------------------------------------------------------------
  ; Exit quietly

    (princ)

)

